@@ -121,6 +121,9 @@ enum nf_conntrack_attr {
 	ATTR_SCTP_VTAG_ORIG,			/* u32 bits */
 	ATTR_SCTP_VTAG_REPL,			/* u32 bits */
 	ATTR_HELPER_NAME,			/* string (30 bytes max) */
+	ATTR_DCCP_STATE = 56,			/* u8 bits */
+	ATTR_DCCP_ROLE,				/* u8 bits */
+	ATTR_DCCP_HANDSHAKE_SEQ,		/* u64 bits */
 	ATTR_MAX
 };
 
@@ -231,6 +234,18 @@ extern int nfct_callback_register(struct nfct_handle *h,
 
 extern void nfct_callback_unregister(struct nfct_handle *h);
 
+/* register / unregister callback: extended version including netlink header */
+
+extern int nfct_callback_register2(struct nfct_handle *h,
+				   enum nf_conntrack_msg_type type,
+				   int (*cb)(const struct nlmsghdr *nlh,
+				   	     enum nf_conntrack_msg_type type,
+				  	     struct nf_conntrack *ct,
+					     void *data),
+				   void *data);
+
+extern void nfct_callback_unregister2(struct nfct_handle *h);
+
 /* callback verdict */
 enum {
 	NFCT_CB_FAILURE = -1,   /* failure */
@@ -256,6 +271,10 @@ extern void nfct_set_attr_u32(struct nf_conntrack *ct,
 			      const enum nf_conntrack_attr type,
 			      u_int32_t value);
 
+extern void nfct_set_attr_u64(struct nf_conntrack *ct,
+			      const enum nf_conntrack_attr type,
+			      u_int64_t value);
+
 /* getter */
 extern const void *nfct_get_attr(const struct nf_conntrack *ct,
 				 const enum nf_conntrack_attr type);
@@ -269,6 +288,9 @@ extern u_int16_t nfct_get_attr_u16(const struct nf_conntrack *ct,
 extern u_int32_t nfct_get_attr_u32(const struct nf_conntrack *ct,
 				   const enum nf_conntrack_attr type);
 
+extern u_int64_t nfct_get_attr_u64(const struct nf_conntrack *ct,
+				   const enum nf_conntrack_attr type);
+
 /* checker */
 extern int nfct_attr_is_set(const struct nf_conntrack *ct,
 			    const enum nf_conntrack_attr type);
@@ -402,12 +424,18 @@ struct nfct_filter_ipv4 {
 	u_int32_t addr;
 	u_int32_t mask;
 };
+struct nfct_filter_ipv6 {
+	u_int32_t addr[4];
+	u_int32_t mask[4];
+};
 
 enum nfct_filter_attr {
 	NFCT_FILTER_L4PROTO = 0,	/* u_int32_t */
 	NFCT_FILTER_L4PROTO_STATE,	/* struct nfct_filter_proto */
 	NFCT_FILTER_SRC_IPV4,		/* struct nfct_filter_ipv4 */
 	NFCT_FILTER_DST_IPV4,		/* struct nfct_filter_ipv4 */
+	NFCT_FILTER_SRC_IPV6,		/* struct nfct_filter_ipv6 */
+	NFCT_FILTER_DST_IPV6,		/* struct nfct_filter_ipv6 */
 	NFCT_FILTER_MAX
 };
 
@@ -491,6 +519,17 @@ extern int nfexp_callback_register(struct nfct_handle *h,
 
 extern void nfexp_callback_unregister(struct nfct_handle *h);
 
+/* register / unregister callback: extended version including netlink header */
+extern int nfexp_callback_register2(struct nfct_handle *h,
+				    enum nf_conntrack_msg_type type,
+				    int (*cb)(const struct nlmsghdr *nlh,
+				    	      enum nf_conntrack_msg_type type,
+					      struct nf_expect *exp,
+					      void *data),
+				    void *data);
+
+extern void nfexp_callback_unregister2(struct nfct_handle *h);
+
 /* setter */
 extern void nfexp_set_attr(struct nf_expect *exp,
 			   const enum nf_expect_attr type,
@@ -616,291 +655,12 @@ enum ip_conntrack_status {
 /* Be liberal in window checking */
 #define IP_CT_TCP_FLAG_BE_LIBERAL               0x08
 
-/* 
- * Old deprecated API, its use for new applications is *strongly discouraged* 
- */
-
-/*
- * In case that the user doesn't want to do some kind
- * of action against a conntrack based on its ID 
- */
-#define NFCT_ANY_ID 0
-
-union nfct_l4 {
-	/* Add other protocols here. */
-	u_int16_t all;
-	struct {
-		u_int16_t port;
-	} tcp;
-	struct {
-		u_int16_t port;
-	} udp;
-	struct {
-		u_int8_t type, code;
-		u_int16_t id;
-	} icmp;
-	struct {
-		u_int16_t port;
-	} sctp;
-};
-
-union nfct_address {
-	u_int32_t v4;
-	u_int32_t v6[4];
-};
-
-struct nfct_tuple {
-	union nfct_address src;
-	union nfct_address dst;
-
-	u_int8_t l3protonum;
-	u_int8_t protonum;
-	union nfct_l4 l4src;
-	union nfct_l4 l4dst;
-};
-
-union nfct_protoinfo {
-	struct {
-		u_int8_t state;
-	} tcp;
-};
-
-struct nfct_counters {
-	u_int64_t packets;
-	u_int64_t bytes;
-};
-
-struct nfct_nat {
-	u_int32_t min_ip, max_ip;
-	union nfct_l4 l4min, l4max;
-};
-
+/* WARNING: do not use these constants in new applications, we keep them here
+ * to avoid breaking backward compatibility. */
 #define NFCT_DIR_ORIGINAL 0
 #define NFCT_DIR_REPLY 1
 #define NFCT_DIR_MAX NFCT_DIR_REPLY+1
 
-struct nfct_conntrack {
-	struct nfct_tuple tuple[NFCT_DIR_MAX];
-	
-	u_int32_t 	timeout;
-	u_int32_t	mark;
-	u_int32_t 	status;
-	u_int32_t	use;
-	u_int32_t	id;
-
-	union nfct_protoinfo protoinfo;
-	struct nfct_counters counters[NFCT_DIR_MAX];
-	struct nfct_nat nat;
-};
-
-struct nfct_expect {
-	struct nfct_tuple master;
-	struct nfct_tuple tuple;
-	struct nfct_tuple mask;
-	u_int32_t timeout;
-	u_int32_t id;
-	u_int16_t expectfn_queue_id;
-};
-
-struct nfct_conntrack_compare {
-	struct nfct_conntrack *ct;
-	unsigned int flags;
-	unsigned int l3flags;
-	unsigned int l4flags;
-};
-
-enum {
-	NFCT_STATUS_BIT = 0,
-	NFCT_STATUS = (1 << NFCT_STATUS_BIT),
-	
-	NFCT_PROTOINFO_BIT = 1,
-	NFCT_PROTOINFO = (1 << NFCT_PROTOINFO_BIT),
-
-	NFCT_TIMEOUT_BIT = 2,
-	NFCT_TIMEOUT = (1 << NFCT_TIMEOUT_BIT),
-
-	NFCT_MARK_BIT = 3,
-	NFCT_MARK = (1 << NFCT_MARK_BIT),
-
-	NFCT_COUNTERS_ORIG_BIT = 4,
-	NFCT_COUNTERS_ORIG = (1 << NFCT_COUNTERS_ORIG_BIT),
-
-	NFCT_COUNTERS_RPLY_BIT = 5,
-	NFCT_COUNTERS_RPLY = (1 << NFCT_COUNTERS_RPLY_BIT),
-
-	NFCT_USE_BIT = 6,
-	NFCT_USE = (1 << NFCT_USE_BIT),
-
-	NFCT_ID_BIT = 7,
-	NFCT_ID = (1 << NFCT_ID_BIT)
-};
-
-enum {
-	NFCT_MSG_UNKNOWN,
-	NFCT_MSG_NEW,
-	NFCT_MSG_UPDATE,
-	NFCT_MSG_DESTROY
-};
-
-typedef int (*nfct_callback)(void *arg, unsigned int flags, int, void *data);
-
-/*
- * [Allocate|free] a conntrack
- */
-extern __attribute__((deprecated))
-struct nfct_conntrack *
-nfct_conntrack_alloc(struct nfct_tuple *orig, struct nfct_tuple *reply,
-		     u_int32_t timeout, union nfct_protoinfo *proto,
-		     u_int32_t status, u_int32_t mark,
-		     u_int32_t id, struct nfct_nat *range);
-extern __attribute__((deprecated))
-void nfct_conntrack_free(struct nfct_conntrack *ct);
-
-/*
- * [Allocate|free] an expectation
- */
-extern __attribute__((deprecated))
-struct nfct_expect *
-nfct_expect_alloc(struct nfct_tuple *master, struct nfct_tuple *tuple,
-		  struct nfct_tuple *mask, u_int32_t timeout, 
-		  u_int32_t id);
-
-extern __attribute__((deprecated))
-void nfct_expect_free(struct nfct_expect *exp);
-
-
-/*
- * [Register|unregister] callbacks
- */
-extern __attribute__((deprecated)) 
-void nfct_register_callback(struct nfct_handle *cth,
-			    nfct_callback callback, void *data);
-
-extern __attribute__((deprecated))
-void nfct_unregister_callback(struct nfct_handle *cth);
-
-/*
- * callback displayers
- */
-extern __attribute__((deprecated)) 
-int nfct_default_conntrack_display(void *, unsigned int, int, void *); 
-extern __attribute__((deprecated))
-int nfct_default_conntrack_display_id(void *, unsigned int, int, void *);
-extern __attribute__((deprecated))
-int nfct_default_expect_display(void *, unsigned int, int, void *);
-extern __attribute__((deprecated))
-int nfct_default_expect_display_id(void *, unsigned int, int, void *);
-extern __attribute__((deprecated))
-int nfct_default_conntrack_event_display(void *, unsigned int, int, void *);
-
-/*
- * [Create|update|get|destroy] conntracks
- */
-extern __attribute__((deprecated))
-int nfct_create_conntrack(struct nfct_handle *cth, 
-			  struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_update_conntrack(struct nfct_handle *cth,
-			  struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_delete_conntrack(struct nfct_handle *cth, 
-			  struct nfct_tuple *tuple, int dir, 
-			  u_int32_t id);
-extern __attribute__((deprecated))
-int nfct_get_conntrack(struct nfct_handle *cth, 
-		       struct nfct_tuple *tuple, int dir,
-		       u_int32_t id); 
-/*
- * Conntrack table dumping & zeroing
- */
-extern __attribute__((deprecated))
-int nfct_dump_conntrack_table(struct nfct_handle *cth, int family);
-extern __attribute__((deprecated))
-int nfct_dump_conntrack_table_reset_counters(struct nfct_handle *cth, 
-					     int family);
-
-/*
- * Conntrack event notification
- */
-extern __attribute__((deprecated))
-int nfct_event_conntrack(struct nfct_handle *cth); 
-
-/*
- * Conntrack printing functions
- */
-extern __attribute__((deprecated))
-int nfct_sprintf_conntrack(char *buf, struct nfct_conntrack *ct, 
-			   unsigned int flags);
-extern __attribute__((deprecated))
-int nfct_sprintf_conntrack_id(char *buf, struct nfct_conntrack *ct,
-			      unsigned int flags);
-extern __attribute__((deprecated))
-int nfct_sprintf_address(char *buf, struct nfct_tuple *t);
-extern __attribute__((deprecated))
-int nfct_sprintf_proto(char *buf, struct nfct_tuple *t);
-extern __attribute__((deprecated))
-int nfct_sprintf_protoinfo(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_timeout(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_protocol(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_status_assured(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_status_seen_reply(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_counters(char *buf, struct nfct_conntrack *ct, int dir);
-extern __attribute__((deprecated))
-int nfct_sprintf_mark(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_use(char *buf, struct nfct_conntrack *ct);
-extern __attribute__((deprecated))
-int nfct_sprintf_id(char *buf, u_int32_t id);
-
-/*
- * Conntrack comparison
- */
-extern __attribute__((deprecated))
-int nfct_conntrack_compare(struct nfct_conntrack *ct1, 
-			   struct nfct_conntrack *ct2,
-			   struct nfct_conntrack_compare *cmp);
-
-/* 
- * Expectations
- */
-extern __attribute__((deprecated))
-int nfct_dump_expect_list(struct nfct_handle *cth, int family);
-extern __attribute__((deprecated))
-int nfct_flush_conntrack_table(struct nfct_handle *cth, int family);
-extern __attribute__((deprecated))
-int nfct_get_expectation(struct nfct_handle *cth, 
-			 struct nfct_tuple *tuple,
-			 u_int32_t id);
-extern __attribute__((deprecated))
-int nfct_create_expectation(struct nfct_handle *cth, struct nfct_expect *);
-extern __attribute__((deprecated))
-int nfct_delete_expectation(struct nfct_handle *cth,
-			    struct nfct_tuple *tuple, u_int32_t id);
-extern __attribute__((deprecated))
-int nfct_event_expectation(struct nfct_handle *cth);
-extern __attribute__((deprecated))
-int nfct_flush_expectation_table(struct nfct_handle *cth, int family);
-
-/*
- * expectation printing functions
- */
-extern __attribute__((deprecated))
-int nfct_sprintf_expect(char *buf, struct nfct_expect *exp);
-extern __attribute__((deprecated))
-int nfct_sprintf_expect_id(char *buf, struct nfct_expect *exp);
-
-/*
- * low-level functions for libnetfilter_cthelper
- */
-extern __attribute__((deprecated))
-void nfct_build_tuple(struct nfnlhdr *req, int size, 
-		      struct nfct_tuple *t, int type);
-
 #ifdef __cplusplus
 }
 #endif
