@@ -18,7 +18,7 @@
  * In case of success, this function returns a valid pointer to a memory blob,
  * otherwise NULL is returned and errno is set appropiately.
  */
-struct nf_conntrack *nfct_new()
+struct nf_conntrack *nfct_new(void)
 {
 	struct nf_conntrack *ct;
 
@@ -66,7 +66,7 @@ size_t nfct_sizeof(const struct nf_conntrack *ct)
  * does although _this could change in the future_. Therefore, do not assume
  * that nfct_sizeof(ct) == nfct_maxsize().
  */
-size_t nfct_maxsize()
+size_t nfct_maxsize(void)
 {
 	return sizeof(struct nf_conntrack);
 }
@@ -199,6 +199,80 @@ void nfct_callback_unregister(struct nfct_handle *h)
 	h->nfnl_cb.attr_count = 0;
 }
 
+/**
+ * nf_callback_register2 - register a callback
+ * @h: library handler
+ * @cb: callback used to process conntrack received
+ * @data: data used by the callback, if any.
+ *
+ * This function register a callback to handle the conntrack received, 
+ * in case of error -1 is returned and errno is set appropiately, otherwise
+ * 0 is returned.
+ *
+ * Note that the data parameter is optional, if you do not want to pass any
+ * data to your callback, then use NULL.
+ *
+ * NOTICE: The difference with nf_callback_register() is that this function
+ * uses the new callback interface that includes the Netlink header.
+ *
+ * WARNING: Don't mix nf_callback_register() and nf_callback_register2()
+ * calls, use only once at a time.
+ */
+int nfct_callback_register2(struct nfct_handle *h,
+			    enum nf_conntrack_msg_type type,
+			    int (*cb)(const struct nlmsghdr *nlh,
+			    	      enum nf_conntrack_msg_type type,
+				      struct nf_conntrack *ct, 
+				      void *data),
+			   void *data)
+{
+	struct __data_container *container;
+
+	assert(h != NULL);
+
+	container = calloc(sizeof(struct __data_container), 1);
+	if (container == NULL)
+		return -1;
+
+	h->cb2 = cb;
+	container->h = h;
+	container->type = type;
+	container->data = data;
+
+	h->nfnl_cb.call = __callback;
+	h->nfnl_cb.data = container;
+	h->nfnl_cb.attr_count = CTA_MAX;
+
+	nfnl_callback_register(h->nfnlssh_ct, 
+			       IPCTNL_MSG_CT_NEW,
+			       &h->nfnl_cb);
+
+	nfnl_callback_register(h->nfnlssh_ct,
+			       IPCTNL_MSG_CT_DELETE,
+			       &h->nfnl_cb);
+
+	return 0;
+}
+
+/**
+ * nfct_callback_unregister2 - unregister a callback
+ * @h: library handler
+ */
+void nfct_callback_unregister2(struct nfct_handle *h)
+{
+	assert(h != NULL);
+
+	nfnl_callback_unregister(h->nfnlssh_ct, IPCTNL_MSG_CT_NEW);
+	nfnl_callback_unregister(h->nfnlssh_ct, IPCTNL_MSG_CT_DELETE);
+
+	h->cb2 = NULL;
+	free(h->nfnl_cb.data);
+
+	h->nfnl_cb.call = NULL;
+	h->nfnl_cb.data = NULL;
+	h->nfnl_cb.attr_count = 0;
+}
+
 /**
  * nfct_set_attr - set the value of a certain conntrack attribute
  * @ct: pointer to a valid conntrack
@@ -266,6 +340,19 @@ void nfct_set_attr_u32(struct nf_conntrack *ct,
 	nfct_set_attr(ct, type, &value);
 }
 
+/**
+ * nfct_set_attr_u64 - set the value of a certain conntrack attribute
+ * @ct: pointer to a valid conntrack
+ * @type: attribute type
+ * @value: unsigned 64 bits attribute value
+ */
+void nfct_set_attr_u64(struct nf_conntrack *ct,
+		       const enum nf_conntrack_attr type, 
+		       u_int64_t value)
+{
+	nfct_set_attr(ct, type, &value);
+}
+
 /**
  * nfct_get_attr - get a conntrack attribute
  * ct: pointer to a valid conntrack
@@ -342,6 +429,22 @@ u_int32_t nfct_get_attr_u32(const struct nf_conntrack *ct,
 	return ret == NULL ? 0 : *ret;
 }
 
+/**
+ * nfct_get_attr_u64 - get attribute of unsigned 32-bits long
+ * @ct: pointer to a valid conntrack
+ * @type: attribute type
+ *
+ * Returns the value of the requested attribute, if the attribute is not 
+ * set, 0 is returned. In order to check if the attribute is set or not,
+ * use nfct_attr_is_set.
+ */
+u_int64_t nfct_get_attr_u64(const struct nf_conntrack *ct,
+			    const enum nf_conntrack_attr type)
+{
+	const u_int64_t *ret = nfct_get_attr(ct, type);
+	return ret == NULL ? 0 : *ret;
+}
+
 /**
  * nfct_attr_is_set - check if a certain attribute is set
  * @ct: pointer to a valid conntrack object
