@@ -7,44 +7,6 @@
 
 #include "internal/internal.h"
 
-static char *proto2str[IPPROTO_MAX] = {
-	[IPPROTO_TCP] = "tcp",
-        [IPPROTO_UDP] = "udp",
-        [IPPROTO_UDPLITE] = "udplite",
-        [IPPROTO_ICMP] = "icmp",
-        [IPPROTO_ICMPV6] = "icmpv6",
-        [IPPROTO_SCTP] = "sctp"
-};
-
-static char *l3proto2str[AF_MAX] = {
-	[AF_INET] = "ipv4",
-	[AF_INET6] = "ipv6"
-};
-
-static const char *states[] = {
-	"NONE",
-	"SYN_SENT",
-	"SYN_RECV",
-	"ESTABLISHED",
-	"FIN_WAIT",
-	"CLOSE_WAIT",
-	"LAST_ACK",
-	"TIME_WAIT",
-	"CLOSE",
-	"LISTEN"
-};
-
-static const char *sctp_states[] = {
-	"NONE",
-	"CLOSED",
-	"COOKIE_WAIT",
-	"COOKIE_ECHOED",
-	"ESTABLISHED",
-	"SHUTDOWN_SENT",
-	"SHUTDOWN_RECD",
-	"SHUTDOWN_ACK_SENT",
-};
-
 static int __snprintf_l3protocol(char *buf,
 				 unsigned int len,
 				 const struct nf_conntrack *ct)
@@ -65,29 +27,46 @@ int __snprintf_protocol(char *buf,
 		 ct->tuple[__DIR_ORIG].protonum));
 }
 
-int __snprintf_timeout(char *buf,
-		       unsigned int len,
-		       const struct nf_conntrack *ct)
+static int __snprintf_timeout(char *buf,
+			      unsigned int len,
+			      const struct nf_conntrack *ct)
 {
 	return snprintf(buf, len, "%u ", ct->timeout);
 }
 
-int __snprintf_protoinfo(char *buf, 
-			 unsigned int len,
-			 const struct nf_conntrack *ct)
+static int __snprintf_protoinfo(char *buf,
+				unsigned int len,
+				const struct nf_conntrack *ct)
 {
-	return snprintf(buf, len, "%s ", states[ct->protoinfo.tcp.state]);
-}
-int __snprintf_protoinfo_sctp(char *buf, 
-			      unsigned int len,
-			      const struct nf_conntrack *ct)
-{
-	return snprintf(buf, len, "%s ", sctp_states[ct->protoinfo.sctp.state]);
+	return snprintf(buf, len, "%s ",
+			ct->protoinfo.tcp.state < TCP_CONNTRACK_MAX ?
+			states[ct->protoinfo.tcp.state] :
+			states[TCP_CONNTRACK_NONE]);
 }
 
-int __snprintf_address_ipv4(char *buf,
-			    unsigned int len,
-			    const struct __nfct_tuple *tuple)
+static int __snprintf_protoinfo_sctp(char *buf,
+				     unsigned int len,
+				     const struct nf_conntrack *ct)
+{
+	return snprintf(buf, len, "%s ",
+			ct->protoinfo.sctp.state < SCTP_CONNTRACK_MAX ?
+			sctp_states[ct->protoinfo.sctp.state] :
+			sctp_states[SCTP_CONNTRACK_NONE]);
+}
+
+static int __snprintf_protoinfo_dccp(char *buf,
+				     unsigned int len,
+				     const struct nf_conntrack *ct)
+{
+	return snprintf(buf, len, "%s ",
+			ct->protoinfo.dccp.state < DCCP_CONNTRACK_MAX ?
+			sctp_states[ct->protoinfo.dccp.state] :
+			sctp_states[DCCP_CONNTRACK_NONE]);
+}
+
+static int __snprintf_address_ipv4(char *buf,
+				   unsigned int len,
+				   const struct __nfct_tuple *tuple)
 {
 	int ret, size = 0, offset = 0;
 	struct in_addr src = { .s_addr = tuple->src.v4 };
@@ -102,9 +81,9 @@ int __snprintf_address_ipv4(char *buf,
 	return size;
 }
 
-int __snprintf_address_ipv6(char *buf, 
-			    unsigned int len,
-			    const struct __nfct_tuple *tuple)
+static int __snprintf_address_ipv6(char *buf,
+				   unsigned int len,
+				   const struct __nfct_tuple *tuple)
 {
 	int ret, size = 0, offset = 0;
 	struct in6_addr src;
@@ -158,11 +137,16 @@ int __snprintf_proto(char *buf,
 	case IPPROTO_UDP:
 	case IPPROTO_UDPLITE:
 	case IPPROTO_SCTP:
+	case IPPROTO_DCCP:
 		return snprintf(buf, len, "sport=%u dport=%u ",
 			        ntohs(tuple->l4src.tcp.port),
 			        ntohs(tuple->l4dst.tcp.port));
 		break;
-
+	case IPPROTO_GRE:
+		return snprintf(buf, len, "srckey=0x%x dstkey=0x%x ",
+			        ntohs(tuple->l4src.all),
+			        ntohs(tuple->l4dst.all));
+		break;
 	case IPPROTO_ICMP:
 	case IPPROTO_ICMPV6:
 		/* The ID only makes sense some ICMP messages but we want to
@@ -177,9 +161,9 @@ int __snprintf_proto(char *buf,
 	return size;
 }
 
-int __snprintf_status_assured(char *buf,
-			      unsigned int len,
-			      const struct nf_conntrack *ct)
+static int __snprintf_status_assured(char *buf,
+				     unsigned int len,
+				     const struct nf_conntrack *ct)
 {
 	int size = 0;
 	
@@ -189,9 +173,9 @@ int __snprintf_status_assured(char *buf,
 	return size;
 }
 
-int __snprintf_status_not_seen_reply(char *buf,
-				     unsigned int len,
-				     const struct nf_conntrack *ct)
+static int __snprintf_status_not_seen_reply(char *buf,
+					    unsigned int len,
+					    const struct nf_conntrack *ct)
 {
 	int size = 0;
 	
@@ -201,10 +185,10 @@ int __snprintf_status_not_seen_reply(char *buf,
 	return size;
 }
 
-int __snprintf_counters(char *buf, 
-		        unsigned int len, 
-		        const struct nf_conntrack *ct,
-		        int dir)
+static int __snprintf_counters(char *buf,
+			       unsigned int len,
+			       const struct nf_conntrack *ct,
+			       int dir)
 {
 	return (snprintf(buf, len, "packets=%llu bytes=%llu ",
 			 (unsigned long long) ct->counters[dir].packets,
@@ -282,6 +266,11 @@ int __snprintf_conntrack_default(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
+	if (test_bit(ATTR_DCCP_STATE, ct->set)) {
+		ret = __snprintf_protoinfo_dccp(buf+offset, len, ct);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
 	ret = __snprintf_address(buf+offset, len, &ct->tuple[__DIR_ORIG]);
 	BUFFER_SIZE(ret, size, len, offset);
 
