@@ -40,9 +40,7 @@
  *		</counters>
  * 	</meta>
  * 	<meta direction="independent">
- *	 	<layer4>
- * 			<state>ESTABLISHED</state>
- * 		</layer4>
+ * 		<state>ESTABLISHED</state>
  * 		<timeout>100</timeout>
  * 		<mark>1</mark>
  * 		<secmark>0</secmark>
@@ -53,30 +51,17 @@
  * </flow>
  */
 
-static char *proto2str[IPPROTO_MAX] = {
-	[IPPROTO_TCP] = "tcp",
-        [IPPROTO_UDP] = "udp",
-        [IPPROTO_UDPLITE] = "udplite",
-        [IPPROTO_ICMP] = "icmp",
-        [IPPROTO_ICMPV6] = "icmp6",
-        [IPPROTO_SCTP] = "sctp"
-};
-static char *l3proto2str[AF_MAX] = {
-	[AF_INET] = "ipv4",
-	[AF_INET6] = "ipv6"
-};
-
 enum {
 	__ADDR_SRC = 0,
 	__ADDR_DST,
 };
 
-static char *__proto2str(u_int8_t protonum)
+static const char *__proto2str(u_int8_t protonum)
 {
 	return proto2str[protonum] ? proto2str[protonum] : "unknown";
 }
 
-static char *__l3proto2str(u_int8_t protonum)
+static const char *__l3proto2str(u_int8_t protonum)
 {
 	return l3proto2str[protonum] ? l3proto2str[protonum] : "unknown";
 }
@@ -167,6 +152,7 @@ static int __snprintf_proto_xml(char *buf,
 	case IPPROTO_UDP:
 	case IPPROTO_UDPLITE:
 	case IPPROTO_SCTP:
+	case IPPROTO_DCCP:
 		if (type == __ADDR_SRC) {
 			ret = snprintf(buf, len, "<sport>%u</sport>", 
 				       ntohs(tuple->l4src.tcp.port));
@@ -177,6 +163,17 @@ static int __snprintf_proto_xml(char *buf,
 			BUFFER_SIZE(ret, size, len, offset);
 		}
 		break;
+	case IPPROTO_GRE:
+		if (type == __ADDR_SRC) {
+			ret = snprintf(buf, len, "<srckey>0x%x</srckey>", 
+				       ntohs(tuple->l4src.all));
+			BUFFER_SIZE(ret, size, len, offset);
+		} else {
+			ret = snprintf(buf, len, "<dstkey>0x%x</dstkey>",
+				       ntohs(tuple->l4dst.all));
+			BUFFER_SIZE(ret, size, len, offset);
+		}
+		break;
 	}
 
 	return ret;
@@ -292,7 +289,10 @@ int __snprintf_conntrack_xml(char *buf,
 	ret = __snprintf_tuple_xml(buf+offset, len, ct, __DIR_REPL);
 	BUFFER_SIZE(ret, size, len, offset);
 
-	if (test_bit(ATTR_TIMEOUT, ct->set) ||
+	if (test_bit(ATTR_TCP_STATE, ct->set) ||
+	    test_bit(ATTR_SCTP_STATE, ct->set) ||
+	    test_bit(ATTR_DCCP_STATE, ct->set) ||
+	    test_bit(ATTR_TIMEOUT, ct->set) ||
 	    test_bit(ATTR_MARK, ct->set) ||
 	    test_bit(ATTR_USE, ct->set) ||
 	    test_bit(ATTR_STATUS, ct->set)) {
@@ -301,6 +301,30 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
+	if (test_bit(ATTR_TCP_STATE, ct->set)) {
+		ret = snprintf(buf+offset, len, "<state>%s</state>",
+			       ct->protoinfo.tcp.state < TCP_CONNTRACK_MAX ?
+			       states[ct->protoinfo.tcp.state] :
+			       states[TCP_CONNTRACK_NONE]);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+	if (test_bit(ATTR_SCTP_STATE, ct->set)) {
+		ret = snprintf(buf+offset, len, "<state>%s</state>",
+			       ct->protoinfo.sctp.state < SCTP_CONNTRACK_MAX ?
+			       states[ct->protoinfo.sctp.state] :
+			       states[SCTP_CONNTRACK_NONE]);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+	if (test_bit(ATTR_DCCP_STATE, ct->set)) {
+		ret = snprintf(buf+offset, len, "<state>%s</state>",
+			       ct->protoinfo.sctp.state < DCCP_CONNTRACK_MAX ?
+			       states[ct->protoinfo.dccp.state] :
+			       states[DCCP_CONNTRACK_NONE]);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
 	if (test_bit(ATTR_TIMEOUT, ct->set)) {
 		ret = snprintf(buf+offset, len,
 				"<timeout>%u</timeout>", ct->timeout);
@@ -340,7 +364,10 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_TIMEOUT, ct->set) ||
+	if (test_bit(ATTR_TCP_STATE, ct->set) ||
+	    test_bit(ATTR_SCTP_STATE, ct->set) ||
+	    test_bit(ATTR_DCCP_STATE, ct->set) ||
+	    test_bit(ATTR_TIMEOUT, ct->set) ||
 	    test_bit(ATTR_MARK, ct->set) ||
 	    test_bit(ATTR_USE, ct->set) ||
 	    test_bit(ATTR_STATUS, ct->set)) {
