@@ -18,7 +18,7 @@
  * In case of success, this function returns a valid pointer to a memory blob,
  * otherwise NULL is returned and errno is set appropiately.
  */
-struct nf_expect *nfexp_new()
+struct nf_expect *nfexp_new(void)
 {
 	struct nf_expect *exp;
 
@@ -66,7 +66,7 @@ size_t nfexp_sizeof(const struct nf_expect *exp)
  * does although _this could change in the future_. Therefore, do not assume
  * that nfexp_sizeof(exp) == nfexp_maxsize().
  */
-size_t nfexp_maxsize()
+size_t nfexp_maxsize(void)
 {
 	return sizeof(struct nf_expect);
 }
@@ -159,6 +159,81 @@ void nfexp_callback_unregister(struct nfct_handle *h)
 	h->nfnl_cb.attr_count = 0;
 }
 
+/**
+ * nfexp_callback_register2 - register a callback
+ * @h: library handler
+ * @cb: callback used to process expect received
+ * @data: data used by the callback, if any.
+ *
+ * This function register a callback to handle the expect received, 
+ * in case of error -1 is returned and errno is set appropiately, otherwise
+ * 0 is returned.
+ *
+ * Note that the data parameter is optional, if you do not want to pass any
+ * data to your callback, then use NULL.
+ *
+ * NOTICE: The difference with nfexp_callback_register() is that this function
+ * uses the new callback interface that includes the Netlink header.
+ *
+ * WARNING: Don't mix nfexp_callback_register() and nfexp_callback_register2()
+ * calls, use only once at a time.
+ */
+int nfexp_callback_register2(struct nfct_handle *h,
+			     enum nf_conntrack_msg_type type,
+			     int (*cb)(const struct nlmsghdr *nlh,
+			     	       enum nf_conntrack_msg_type type,
+			   	       struct nf_expect *exp, 
+				       void *data),
+			     void *data)
+{
+	struct __data_container *container;
+
+	assert(h != NULL);
+
+	container = malloc(sizeof(struct __data_container));
+	if (!container)
+		return -1;
+	memset(container, 0, sizeof(struct __data_container));
+
+	h->expect_cb2 = cb;
+	container->h = h;
+	container->type = type;
+	container->data = data;
+
+	h->nfnl_cb.call = __expect_callback;
+	h->nfnl_cb.data = container;
+	h->nfnl_cb.attr_count = CTA_EXPECT_MAX;
+
+	nfnl_callback_register(h->nfnlssh_exp, 
+			       IPCTNL_MSG_EXP_NEW,
+			       &h->nfnl_cb);
+
+	nfnl_callback_register(h->nfnlssh_exp,
+			       IPCTNL_MSG_EXP_DELETE,
+			       &h->nfnl_cb);
+
+	return 0;
+}
+
+/**
+ * nfexp_callback_unregister2 - unregister a callback
+ * @h: library handler
+ */
+void nfexp_callback_unregister2(struct nfct_handle *h)
+{
+	assert(h != NULL);
+
+	nfnl_callback_unregister(h->nfnlssh_exp, IPCTNL_MSG_EXP_NEW);
+	nfnl_callback_unregister(h->nfnlssh_exp, IPCTNL_MSG_EXP_DELETE);
+
+	h->expect_cb2 = NULL;
+	free(h->nfnl_cb.data);
+
+	h->nfnl_cb.call = NULL;
+	h->nfnl_cb.data = NULL;
+	h->nfnl_cb.attr_count = 0;
+}
+
 /**
  * nfexp_set_attr - set the value of a certain expect attribute
  * @exp: pointer to a valid expect 
@@ -358,12 +433,12 @@ int nfexp_attr_unset(struct nf_expect *exp,
  * On error, -1 is returned and errno is appropiately set.
  * On success, 0 is returned.
  */
-int nfexp_build_expect(struct nfnl_subsys_handle *ssh,
-		       void *req,
-		       size_t size,
-		       u_int16_t type,
-		       u_int16_t flags,
-		       const struct nf_expect *exp)
+static int nfexp_build_expect(struct nfnl_subsys_handle *ssh,
+			      void *req,
+			      size_t size,
+			      u_int16_t type,
+			      u_int16_t flags,
+			      const struct nf_expect *exp)
 {
 	assert(ssh != NULL);
 	assert(req != NULL);
@@ -402,11 +477,11 @@ int nfexp_build_expect(struct nfnl_subsys_handle *ssh,
  * On success, 0 is returned. On error, -1 is returned and errno is set
  * appropiately.
  */
-int nfexp_build_query(struct nfnl_subsys_handle *ssh,
-		      const enum nf_conntrack_query qt,
-		      const void *data,
-		      void *buffer,
-		      unsigned int size)
+static int nfexp_build_query(struct nfnl_subsys_handle *ssh,
+			     const enum nf_conntrack_query qt,
+			     const void *data,
+			     void *buffer,
+			     unsigned int size)
 {
 	struct nfnlhdr *req = buffer;
 	const u_int8_t *family = data;
@@ -464,9 +539,9 @@ int nfexp_build_query(struct nfnl_subsys_handle *ssh,
  * the message received is not of the requested type then 0 is returned, 
  * otherwise this function returns the message type parsed.
  */
-int nfexp_parse_expect(enum nf_conntrack_msg_type type,
-		       const struct nlmsghdr *nlh,
-		       struct nf_expect *exp)
+static int nfexp_parse_expect(enum nf_conntrack_msg_type type,
+			      const struct nlmsghdr *nlh,
+			      struct nf_expect *exp)
 {
 	unsigned int flags;
 	int len = nlh->nlmsg_len;
