@@ -18,8 +18,6 @@ struct nfct_handle *nfct_open_nfnl(struct nfnl_handle *nfnlh,
 {
 	struct nfct_handle *cth;
 
-	deprecated_backward_support();
-
 	cth = (struct nfct_handle *) malloc(sizeof(struct nfct_handle));
 	if (!cth)
 		return NULL;
@@ -90,6 +88,9 @@ int nfct_close(struct nfct_handle *cth)
 
 	/* required by the new API */
 	cth->cb = NULL;
+	cth->cb2 = NULL;
+	cth->expect_cb = NULL;
+	cth->expect_cb2 = NULL;
 	free(cth->nfnl_cb.data);
 
 	cth->nfnl_cb.call = NULL; 
