@@ -18,10 +18,13 @@ int main()
 
 	ret = nfexp_query(h, NFCT_Q_FLUSH, &family);
 
-	printf("TEST: flush expectation (%d)(%s)\n", ret, strerror(errno));
-
+	printf("TEST: flush expectations ");
 	if (ret == -1)
-		exit(EXIT_FAILURE);
+		printf("(%d)(%s)\n", ret, strerror(errno));
+	else
+		printf("(OK)\n");
 
-	exit(EXIT_SUCCESS);
+	nfct_close(h);
+
+	ret == -1 ? exit(EXIT_FAILURE) : exit(EXIT_SUCCESS);
 }
